==> spread-5.3.0.5-py3-none-any.whl: spread_dashboard/scheduler/views.py <==
import datetime
import json
import sys

import django.http as http
from dateutil import relativedelta
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.translation import gettext as _

from spread_dashboard.responses import Success
from spread_dashboard.scheduler import models


@login_required
def on_api(request, api_name):
    if hasattr(sys.modules[__name__], api_name):
        try:
            return getattr(sys.modules[__name__], api_name)(request)
        except BaseException as ex:
            print(str(ex))
            return http.HttpResponseServerError(str(ex))

    return http.HttpResponseNotAllowed(api_name)


@login_required
def page(request):
    return render(request, 'scheduler/scheduler.html', context=dict(
        root_location=_locations(False),
        selected_id=request.GET.get('location_id', ''),
        commands=json.dumps([cmd['commands'] for cmd in models.SchBase.get_dict().values()]),
        groups=json.dumps([cmd.get('name', None) for cmd in models.SchBase.get_dict().values()])
    ))


def part(request, name):
    return render(request, f'scheduler/{name}.html')


@login_required
def month(request, m=None, y=None):
    if m is None or y is None:
        date = datetime.date.today()
        m = date.month
        y = date.year
    else:
        m = int(m)
        y = int(y)
        date = datetime.date(y, m, 1)
    weeks = []

    day = datetime.date(date.year, date.month, 1)
    holidays = [d.day for d in models.Holiday.objects.filter(date__month=m, date__year=y).values_list('date', flat=True)]

    while day.month == date.month:
        week = []
        for i in range(0, 7):
            classes = []
            if day.month != date.month:
                break
            if day.day == 1 and i < day.weekday():
                week.append('')
            else:
                if day == datetime.date.today():
                    classes.append('current')

                if day.day in holidays:
                    classes.append('weekend')
                else:
                    classes.append('work')

                week.append((' '.join(classes), day.day))
                day += relativedelta.relativedelta(days=1)

        weeks.append(week)

    context = dict(
        name=_('scheduler_month_year') % (
            [None, _('january'), _('february'), _('march'), _('april'),
             _('may'), _('june'), _('july'), _('august'), _('september'),
             _('october'), _('november'), _('december')][m],
            y
        ),
        month=m,
        year=y,
        prev_month=(date - relativedelta.relativedelta(months=1, day=1)).month,
        prev_year=(date - relativedelta.relativedelta(months=1, day=1)).year,
        next_month=(date + relativedelta.relativedelta(months=1, day=1)).month,
        next_year=(date + relativedelta.relativedelta(months=1, day=1)).year,
        weeks=weeks,
        day_names=[
            _('scheduler_monday'),
            _('scheduler_tuesday'),
            _('scheduler_wednesday'),
            _('scheduler_thursday'),
            _('scheduler_friday'),
            _('scheduler_saturday'),
            _('scheduler_sunday')
        ]
    )

    return render(request, 'scheduler/month.html', context=context)


def get_trigger_div(request):
    return render(request, 'scheduler/trigger.html')


def get_action_div(request):
    return render(request, 'scheduler/action.html')


def get_triggers(request):
    return Success([
        dict(
            id=trigger.id,
            time=str(trigger.time),
            actions=[
                action.to_view_data()
                for action in models.SchAction.objects.filter(trigger=trigger)
            ],
        )
        for trigger in models.SchTrigger.objects.filter(
            location=int(request.GET.get('id')),
            type=request.GET.get('trigger')
        )
    ])


def set_enabled(request):
    _id = int(request.GET.get('id'))
    _enabled = request.GET.get('enabled') == 'true'
    model = models.SchLocation.objects.get(id=_id)
    model.enabled = _enabled
    model.save()
    return Success(_enabled)


def status(request):
    loc_id = int(request.GET.get('id'))
    return Success(dict(
        entities=models.SchAction.entities(loc_id),
        enabled=models.SchLocation.objects.get(id=loc_id).enabled
    ))


def set_day(request):
    y = int(request.GET.get('year'))
    m = int(request.GET.get('month'))
    d = int(request.GET.get('day'))

    date = datetime.date(y, m, d)

    try:
        models.Holiday.objects.create(date=date)
        return Success(True)
    except:
        models.Holiday.objects.get(date=date).delete()
        return Success(False)


def set_multi_day(request):
    y = int(request.GET.get('year'))
    # m = int(request.GET.get('month'))
    d = int(request.GET.get('day'))

    last_day = datetime.date(y+1, 1, 1)
    now = datetime.datetime.now().date()

    if y < now.year:
        raise ValueError()
    elif y == now.year:
        date = datetime.date(y, now.month, 1)
    else:
        date = datetime.date(y, 1, 1)

    date += datetime.timedelta(days=d-date.weekday())

    try:
        models.Holiday.objects.get(date=date)
    except:
        days = []
        while date < last_day:
            try:
                # if date >= now:
                days.append(models.Holiday(date=date))
            except:
                pass
            finally:
                date += datetime.timedelta(days=7)

        models.Holiday.objects.bulk_create(days, ignore_conflicts=True)

        result = True
    else:
        try:
            models.Holiday.objects.filter(
                date__range=(date, last_day)
            ).filter(
                date__week_day=(d+2) % 7 if d != 5 else 7
            ).delete()
        except BaseException as ex:
            print(str(ex))
        result = False

    return Success(result)


def set_time(request):
    _loc_id = int(request.GET.get('location'))
    _tr_id = request.GET.get('trigger')
    _type = request.GET.get('type')
    _time = request.GET.get('time')

    if _tr_id:
        model = models.SchTrigger.objects.get(id=int(_tr_id), type=_type)
        model.time = _time
        model.save()
    else:
        model = models.SchTrigger.objects.create(time=_time, location=models.SchLocation.objects.get(id=_loc_id), type=_type)

    return Success(dict(id=model.id, time=model.time))


def remove_trigger(request):
    models.SchTrigger.objects.get(id=int(request.GET.get('trigger'))).delete()
    return Success()


def remove_action(request):
    models.SchAction.objects.get(id=int(request.GET.get('action'))).delete()
    return Success()


def set_action(request):
    return Success(
        models.SchAction.from_view_data(
            e_id=int(request.GET.get('eId')),
            tr_id=int(request.GET.get('trigger')),
            target=int(request.GET.get('target')),
            act_id=request.GET.get('action'),
            index=int(request.GET.get('index')),
            value=request.GET.get('value')
        ).id
    )


def _locations(engineries: bool) -> dict:
    locations = {'level': -1}
    for loc in models.bam.models.Location.objects.all():
        node = locations
        for l_id in loc.navigations:
            node.setdefault('locations', [])
            cl = None
            for _cl in node['locations']:
                if _cl.get('id') == l_id:
                    cl = _cl
                    break

            if cl is None:
                cl = {'id': l_id, 'level': node['level'] + 1}
                node['locations'].append(cl)
            node = cl
            node.setdefault('locations', [])

        node['id'] = loc.id
        node['name'] = loc.name

        try:
            sg = models.bam.models.Subginery.objects.get(locationID=loc.id, type=models.subgineries.Lighting.__name__)

            node['sch'] = models.SchLocation.objects.get(id=sg.id)

            if engineries:
                node['engineries'] = [
                    {
                        'type': e.type,
                        'id': e.id,
                        'name': e.v_name,
                        'level': node['level'] + 1,
                    }
                    for e in models.bam.models.Enginery.objects.filter(name__endswith='*', id__in=sg.engineries)
                ]
        except (models.SchLocation.DoesNotExist, models.bam.models.Subginery.DoesNotExist):
            pass

    return locations['locations'][0]


@login_required
def browser(request):
    entity = None
    selected_id = None
    actions = None
    entity_id = request.GET.get('entity_id', None)

    if entity_id is not None:
        entity_id = int(entity_id)

        try:
            entity = models.bam.models.Subginery.objects.get(id=entity_id)
            try:
                setattr(entity, 'name', models.bam.models.Location.objects.get(id=entity.locationID).v_name)
                selected_id = entity.locationID

            except models.bam.models.Location.DoesNotExist:
                setattr(entity, 'name', ':'.join([str(i) for i in (entity.type, entity_id)]))

        except models.bam.models.Subginery.DoesNotExist:
            try:
                entity = models.bam.models.Location.objects.get(id=entity_id)
                selected_id = entity.id

                try:
                    entity = models.bam.models.Subginery.objects.get(
                        locationID=entity.id,
                        type=models.subgineries.Lighting.__name__
                    )
                    entity_id = entity.id
                except models.bam.models.Subginery.DoesNotExist:
                    pass

            except models.bam.models.Location.DoesNotExist:
                try:
                    entity = models.bam.models.Enginery.objects.get(id=entity_id)
                    selected_id = entity.id

                except models.bam.models.Enginery.DoesNotExist:
                    pass

        if actions is None:
            actions = models.SchAction.objects.filter(topic__contains='/' + str(entity_id) + '/')

        actions = {
            "workday": actions.filter(trigger__type='workday'),
            "holiday": actions.filter(trigger__type='holiday'),
        }

        if entity:
            setattr(entity, 'verbose', models.SchAction.get_dict()[entity.type]['name'])

    return render(request, 'scheduler/browser.html', context=dict(
        root_location=_locations(True),
        entity=entity,
        selected_id=selected_id,
        actions=actions,
        show_selector='iamsuperman' in request.GET
    ))

==> spread-5.4.1.16-py3-none-any.whl: spread_dashboard/scheduler/views.py <==
import datetime
import json
import sys

import django.http as http
from dateutil import relativedelta
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.translation import gettext as _

from spread_dashboard.responses import Success
from spread_dashboard.scheduler import models


@login_required
def on_api(request, api_name):
    if hasattr(sys.modules[__name__], api_name):
        try:
            return getattr(sys.modules[__name__], api_name)(request)
        except BaseException as ex:
            print(str(ex))
            return http.HttpResponseServerError(str(ex))

    return http.HttpResponseNotAllowed(api_name)


@login_required
def page(request):
    return render(request, 'scheduler/scheduler.html', context=dict(
        root_location=_locations(False),
        selected_id=request.GET.get('location_id', ''),
        commands=json.dumps([cmd['commands'] for cmd in models.SchBase.get_dict().values()]),
        groups=json.dumps([cmd.get('name', None) for cmd in models.SchBase.get_dict().values()])
    ))


def part(request, name):
    return render(request, f'scheduler/{name}.html')


@login_required
def month(request, m=None, y=None):
    if m is None or y is None:
        date = datetime.date.today()
        m = date.month
        y = date.year
    else:
        m = int(m)
        y = int(y)
        date = datetime.date(y, m, 1)
    weeks = []

    day = datetime.date(date.year, date.month, 1)
    holidays = [d.day for d in models.Holiday.objects.filter(date__month=m, date__year=y).values_list('date', flat=True)]

    while day.month == date.month:
        week = []
        for i in range(0, 7):
            classes = []
            if day.month != date.month:
                break
            if day.day == 1 and i < day.weekday():
                week.append('')
            else:
                if day == datetime.date.today():
                    classes.append('current')

                if day.day in holidays:
                    classes.append('weekend')
                else:
                    classes.append('work')

                week.append((' '.join(classes), day.day))
                day += relativedelta.relativedelta(days=1)

        weeks.append(week)

    context = dict(
        name=_('scheduler_month_year') % (
            [None, _('january'), _('february'), _('march'), _('april'),
             _('may'), _('june'), _('july'), _('august'), _('september'),
             _('october'), _('november'), _('december')][m],
            y
        ),
        month=m,
        year=y,
        prev_month=(date - relativedelta.relativedelta(months=1, day=1)).month,
        prev_year=(date - relativedelta.relativedelta(months=1, day=1)).year,
        next_month=(date + relativedelta.relativedelta(months=1, day=1)).month,
        next_year=(date + relativedelta.relativedelta(months=1, day=1)).year,
        weeks=weeks,
        day_names=[
            _('scheduler_monday'),
            _('scheduler_tuesday'),
            _('scheduler_wednesday'),
            _('scheduler_thursday'),
            _('scheduler_friday'),
            _('scheduler_saturday'),
            _('scheduler_sunday')
        ]
    )

    return render(request, 'scheduler/month.html', context=context)


def get_trigger_div(request):
    return render(request, 'scheduler/trigger.html')


def get_action_div(request):
    return render(request, 'scheduler/action.html')


def get_triggers(request):
    return Success([
        dict(
            id=trigger.id,
            time=str(trigger.time),
            actions=[
                action.to_view_data()
                for action in models.SchAction.objects.filter(trigger=trigger)
            ],
        )
        for trigger in models.SchTrigger.objects.filter(
            location=int(request.GET.get('id')),
            type=request.GET.get('trigger')
        )
    ])


def set_enabled(request):
    _id = int(request.GET.get('id'))
    _enabled = request.GET.get('enabled') == 'true'
    model = models.SchLocation.objects.get(id=_id)
    model.enabled = _enabled
    model.save()
    return Success(_enabled)


def status(request):
    loc_id = int(request.GET.get('id'))
    return Success(dict(
        entities=models.SchAction.entities(loc_id),
        enabled=models.SchLocation.objects.get(id=loc_id).enabled
    ))


def set_day(request):
    y = int(request.GET.get('year'))
    m = int(request.GET.get('month'))
    d = int(request.GET.get('day'))

    date = datetime.date(y, m, d)

    try:
        models.Holiday.objects.create(date=date)
        return Success(True)
    except:
        models.Holiday.objects.get(date=date).delete()
        return Success(False)


def set_multi_day(request):
    y = int(request.GET.get('year'))
    # m = int(request.GET.get('month'))
    d = int(request.GET.get('day'))

    last_day = datetime.date(y+1, 1, 1)
    now = datetime.datetime.now().date()

    if y < now.year:
        raise ValueError()
    elif y == now.year:
        date = datetime.date(y, now.month, 1)
    else:
        date = datetime.date(y, 1, 1)

    date += datetime.timedelta(days=d-date.weekday())

    try:
        models.Holiday.objects.get(date=date)
    except:
        days = []
        while date < last_day:
            try:
                # if date >= now:
                days.append(models.Holiday(date=date))
            except:
                pass
            finally:
                date += datetime.timedelta(days=7)

        models.Holiday.objects.bulk_create(days, ignore_conflicts=True)

        result = True
    else:
        try:
            models.Holiday.objects.filter(
                date__range=(date, last_day)
            ).filter(
                date__week_day=(d+2) % 7 if d != 5 else 7
            ).delete()
        except BaseException as ex:
            print(str(ex))
        result = False

    return Success(result)


def set_time(request):
    _loc_id = int(request.GET.get('location'))
    _tr_id = request.GET.get('trigger')
    _type = request.GET.get('type')
    _time = request.GET.get('time')

    if _tr_id:
        model = models.SchTrigger.objects.get(id=int(_tr_id), type=_type)
        model.time = _time
        model.save()
    else:
        model = models.SchTrigger.objects.create(time=_time, location=models.SchLocation.objects.get(id=_loc_id), type=_type)

    return Success(dict(id=model.id, time=model.time))


def remove_trigger(request):
    models.SchTrigger.objects.get(id=int(request.GET.get('trigger'))).delete()
    return Success()


def remove_action(request):
    models.SchAction.objects.get(id=int(request.GET.get('action'))).delete()
    return Success()


def set_action(request):
    return Success(
        models.SchAction.from_view_data(
            e_id=int(request.GET.get('eId')),
            tr_id=int(request.GET.get('trigger')),
            target=int(request.GET.get('target')),
            act_id=request.GET.get('action'),
            index=int(request.GET.get('index')),
            value=request.GET.get('value')
        ).id
    )


def _locations(engineries: bool) -> dict:
    locations = {'level': -1}
    for loc in models.bam.models.Location.objects.all():
        node = locations
        for l_id in loc.navigations:
            node.setdefault('locations', [])
            cl = None
            for _cl in node['locations']:
                if _cl.get('id') == l_id:
                    cl = _cl
                    break

            if cl is None:
                cl = {'id': l_id, 'level': node['level'] + 1}
                node['locations'].append(cl)
            node = cl
            node.setdefault('locations', [])

        node['id'] = loc.id
        node['name'] = loc.name

        try:
            sg = models.bam.models.Subginery.objects.get(locationID=loc.id, type=models.subgineries.lighting.Lighting.__name__)

            node['sch'] = models.SchLocation.objects.get(id=sg.id)

            if engineries:
                node['engineries'] = [
                    {
                        'type': e.type,
                        'id': e.id,
                        'name': e.v_name,
                        'level': node['level'] + 1,
                    }
                    for e in models.bam.models.Enginery.objects.filter(name__endswith='*', id__in=sg.engineries)
                ]
        except (models.SchLocation.DoesNotExist, models.bam.models.Subginery.DoesNotExist):
            pass

    return locations['locations'][0]


@login_required
def browser(request):
    entity = None
    selected_id = None
    actions = None
    entity_id = request.GET.get('entity_id', None)

    if entity_id is not None:
        entity_id = int(entity_id)

        try:
            entity = models.bam.models.Subginery.objects.get(id=entity_id)
            try:
                setattr(entity, 'name', models.bam.models.Location.objects.get(id=entity.locationID).v_name)
                selected_id = entity.locationID

            except models.bam.models.Location.DoesNotExist:
                setattr(entity, 'name', ':'.join([str(i) for i in (entity.type, entity_id)]))

        except models.bam.models.Subginery.DoesNotExist:
            try:
                entity = models.bam.models.Location.objects.get(id=entity_id)
                selected_id = entity.id

                try:
                    entity = models.bam.models.Subginery.objects.get(
                        locationID=entity.id,
                        type=models.subgineries.lighting.Lighting.__name__
                    )
                    entity_id = entity.id
                except models.bam.models.Subginery.DoesNotExist:
                    pass

            except models.bam.models.Location.DoesNotExist:
                try:
                    entity = models.bam.models.Enginery.objects.get(id=entity_id)
                    selected_id = entity.id

                except models.bam.models.Enginery.DoesNotExist:
                    pass

        if actions is None:
            actions = models.SchAction.objects.filter(topic__contains='/' + str(entity_id) + '/')

        actions = {
            "workday": actions.filter(trigger__type='workday'),
            "holiday": actions.filter(trigger__type='holiday'),
        }

        if entity:
            setattr(entity, 'verbose', models.SchAction.get_dict()[entity.type]['name'])

    return render(request, 'scheduler/browser.html', context=dict(
        root_location=_locations(True),
        entity=entity,
        selected_id=selected_id,
        actions=actions,
        show_selector='iamsuperman' in request.GET
    ))
